package iiopthread;

import ejb.SessionARemote;
import javax.ejb.EJB;
import javax.naming.InitialContext;

/**
 * Reproduce IIOP thread deadlock problem:
 * 1. Client looks up a number of SFSBs on the server.
 * 2. Client calls SFSB's business method concurrently.
 * 3. SFSB calls another session bean on ANOTHER server.
 * -> when concurrency reaches the first server's default
 *    ORB thread pool's Maximum Pool Size, all EJBs freeze.
 *
 */
@EJB(name = "ejb/SessionABean", beanInterface = SessionARemote.class)
public class Main {

    public static void main(String[] args) throws Exception {
        int invocations = args.length == 0 ? 1 : Integer.parseInt(args[0]);

        SessionARemote[] ejbs = new SessionARemote[invocations];
        InitialContext ic = new InitialContext();
        // pre-lookup SFSB's (which look up the bean in the next server)
        // to prevent deadlock in lookup.
        for (int i = 0; i < ejbs.length; i++) {
            System.out.println((i + 1) + ": Looking up SessionA...");
            System.out.flush();
            ejbs[i] = (SessionARemote) ic.lookup("java:comp/env/ejb/SessionABean");
        }
        System.out.println("");
        System.out.flush();

        Thread[] threadArray = new Thread[invocations];
        int counter = 0;
        for (final SessionARemote sessionABean : ejbs) {
            final int id = counter + 1;
            threadArray[counter] = new Thread() {

                @Override
                public void run() {
                    System.out.println(id + ": Calling sessionABean's call method...");
                    System.out.flush();
                    sessionABean.call(id);
                    System.out.println(id + ": Finished calling sessionABean's call method.");
                    System.out.flush();
                }
            };
            threadArray[counter].start();
            counter++;
        }

        // main waits for all threads to finish.
        for (Thread thread : threadArray) {
            thread.join();
        }
        System.out.println("Main finished.");
    }
}
